@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 ###############################################################################
 # Copyright (c), Forschungszentrum Jülich GmbH, IAS-1/PGI-1, Germany.         #
 #                All rights reserved.                                         #
@@ -13,27 +12,27 @@
 """
 Common functions for converting types to and from XML files
 """
+from __future__ import annotations
 
-from typing import TYPE_CHECKING, Iterable, List, Tuple, Union, Dict, Any, cast
+from typing import Iterable, Any, cast
 try:
-    from typing import Literal
+    from typing import Literal, TypeAlias  #type:ignore
 except ImportError:
-    from typing_extensions import Literal  #type:ignore
+    from typing_extensions import Literal, TypeAlias  #type:ignore
 from lxml import etree
-from logging import Logger
-if TYPE_CHECKING:
-    from masci_tools.io.parsers import fleur_schema
+import logging
+from masci_tools.io.parsers import fleur_schema
 
-BASE_TYPES = Union[Literal['int'], Literal['switch'], Literal['string'], Literal['float'], Literal['float_expression']]
-CONVERTED_TYPES = Union[int, float, bool, str]
+BaseType: TypeAlias = Literal['int', 'switch', 'string', 'float', 'float_expression']
+ConvertedType: TypeAlias = 'int | float | bool | str'
 
 
-def convert_to_xml(value: Union[Any, List[Any]],
-                   schema_dict: 'fleur_schema.SchemaDict',
+def convert_to_xml(value: Any | list[Any],
+                   schema_dict: fleur_schema.SchemaDict,
                    name: str,
                    text: bool = False,
-                   logger: Logger = None,
-                   list_return: bool = False) -> Tuple[Union[str, List[str]], bool]:
+                   logger: logging.Logger | None = None,
+                   list_return: bool = False) -> tuple[str | list[str], bool]:
     """
     Tries to converts a given string to the types specified in the schema_dict.
     First succeeded conversion will be returned
@@ -72,14 +71,14 @@ def convert_to_xml(value: Union[Any, List[Any]],
 
 
 def convert_from_xml(
-    xmlstring: Union[str, List[str]],
-    schema_dict: 'fleur_schema.SchemaDict',
+    xmlstring: str | list[str],
+    schema_dict: fleur_schema.SchemaDict,
     name: str,
     text: bool = False,
-    constants: Dict[str, float] = None,
-    logger: Logger = None,
+    constants: dict[str, float] | None = None,
+    logger: logging.Logger | None = None,
     list_return: bool = False
-) -> Tuple[Union[CONVERTED_TYPES, List[CONVERTED_TYPES], List[Union[CONVERTED_TYPES, List[CONVERTED_TYPES]]]], bool]:
+) -> tuple[ConvertedType | list[ConvertedType] | list[ConvertedType | list[ConvertedType]], bool]:
     """
     Tries to converts a given string to the types specified in the schema_dict.
     First succeeded conversion will be returned
@@ -120,12 +119,12 @@ def convert_from_xml(
 
 
 def convert_from_xml_explicit(
-    xmlstring: Union[str, List[str]],
-    definitions: List['fleur_schema.AttributeType'],
-    constants: Dict[str, float] = None,
-    logger: Logger = None,
+    xmlstring: str | list[str],
+    definitions: list[fleur_schema.AttributeType],
+    constants: dict[str, float] | None = None,
+    logger: logging.Logger | None = None,
     list_return: bool = False
-) -> Tuple[Union[CONVERTED_TYPES, List[CONVERTED_TYPES], List[Union[CONVERTED_TYPES, List[CONVERTED_TYPES]]]], bool]:
+) -> tuple[ConvertedType | list[ConvertedType] | list[ConvertedType | list[ConvertedType]], bool]:
     """
     Tries to converts a given string to the types given in definitions.
     First succeeded conversion will be returned
@@ -146,7 +145,7 @@ def convert_from_xml_explicit(
     if not isinstance(xmlstring, list):
         xmlstring = [xmlstring]
 
-    converted_list: List[Union[CONVERTED_TYPES, List[CONVERTED_TYPES]]] = []
+    converted_list: list[ConvertedType | list[ConvertedType]] = []
     all_success = True
     for text in xmlstring:
 
@@ -172,7 +171,7 @@ def convert_from_xml_explicit(
             all_success = False
             continue
 
-        types: Tuple[BASE_TYPES, ...] = tuple(definition.base_type for definition in text_definitions)  #type:ignore
+        types: tuple[BaseType, ...] = tuple(definition.base_type for definition in text_definitions)
         lengths = {definition.length for definition in text_definitions}
 
         if len(text_definitions) == 1:
@@ -184,7 +183,7 @@ def convert_from_xml_explicit(
         all_success = all_success and suc
 
         if len(converted_text) == 1 and 'unbounded' not in lengths:
-            converted_text = converted_text[0]  #type:ignore
+            converted_text = converted_text[0]
         elif len(converted_text) == 0 and 'unbounded' not in lengths:
             converted_text = ''  #type:ignore
 
@@ -192,16 +191,16 @@ def convert_from_xml_explicit(
 
     ret_value = converted_list
     if len(converted_list) == 1 and not list_return:
-        ret_value = converted_list[0]  #type:ignore
+        ret_value = converted_list[0]
 
     return ret_value, all_success
 
 
-def convert_to_xml_explicit(value: Union[Any, Iterable[Any]],
-                            definitions: List['fleur_schema.AttributeType'],
-                            logger: Logger = None,
+def convert_to_xml_explicit(value: Any | Iterable[Any],
+                            definitions: list[fleur_schema.AttributeType],
+                            logger: logging.Logger | None = None,
                             float_format: str = '.10',
-                            list_return: bool = False) -> Tuple[Union[str, List[str]], bool]:
+                            list_return: bool = False) -> tuple[str | list[str], bool]:
     """
     Tries to convert a given list of values to str for a xml file based on the definitions (length and type).
     First succeeded conversion will be returned
@@ -254,7 +253,7 @@ def convert_to_xml_explicit(value: Union[Any, Iterable[Any]],
             all_success = False
             continue
 
-        types: Tuple[BASE_TYPES, ...] = tuple(definition.base_type for definition in text_definitions)  #type:ignore
+        types: tuple[BaseType, ...] = tuple(definition.base_type for definition in text_definitions)
 
         converted_text, suc = convert_to_xml_single_values(val, types, logger=logger, float_format=float_format)
         all_success = all_success and suc
@@ -268,10 +267,10 @@ def convert_to_xml_explicit(value: Union[Any, Iterable[Any]],
     return ret_value, all_success
 
 
-def convert_from_xml_single_values(xmlstring: Union[str, List[str]],
-                                   possible_types: Tuple[BASE_TYPES, ...],
-                                   constants: Dict[str, float] = None,
-                                   logger: Logger = None) -> Tuple[List[CONVERTED_TYPES], bool]:
+def convert_from_xml_single_values(xmlstring: str | list[str],
+                                   possible_types: tuple[BaseType, ...],
+                                   constants: dict[str, float] | None = None,
+                                   logger: logging.Logger | None = None) -> tuple[list[ConvertedType], bool]:
     """
     Tries to converts a given string attribute to the types given in possible_types.
     First succeeded conversion will be returned
@@ -293,12 +292,12 @@ def convert_from_xml_single_values(xmlstring: Union[str, List[str]],
     if not isinstance(xmlstring, list):
         xmlstring = [xmlstring]
 
-    converted_value: CONVERTED_TYPES
+    converted_value: ConvertedType
     converted_list = []
     all_success = True
     for text in xmlstring:
 
-        exceptions: List[Exception] = []
+        exceptions: list[Exception] = []
         for value_type in possible_types:
             if value_type == 'float':
                 try:
@@ -357,10 +356,10 @@ def convert_from_xml_single_values(xmlstring: Union[str, List[str]],
     return converted_list, all_success
 
 
-def convert_to_xml_single_values(value: Union[Any, Iterable[Any]],
-                                 possible_types: Tuple[BASE_TYPES, ...],
-                                 logger: Logger = None,
-                                 float_format: str = '.10') -> Tuple[List[str], bool]:
+def convert_to_xml_single_values(value: Any | Iterable[Any],
+                                 possible_types: tuple[BaseType, ...],
+                                 logger: logging.Logger | None = None,
+                                 float_format: str = '.10') -> tuple[list[str], bool]:
     """
     Tries to converts a given attributevalue to a string for a xml file according
     to the types given in possible_types.
@@ -385,7 +384,7 @@ def convert_to_xml_single_values(value: Union[Any, Iterable[Any]],
 
     converted_value: str
     converted_list = []
-    exceptions: List[Exception] = []
+    exceptions: list[Exception] = []
     all_success = True
     for val in value:
 
@@ -426,7 +425,7 @@ def convert_to_xml_single_values(value: Union[Any, Iterable[Any]],
     return converted_list, all_success
 
 
-def convert_from_fortran_bool(stringbool: Union[str, bool]) -> bool:
+def convert_from_fortran_bool(stringbool: str | bool) -> bool:
     """
     Converts a string in this case ('T', 'F', or 't', 'f') to True or False
 
@@ -447,10 +446,10 @@ def convert_from_fortran_bool(stringbool: Union[str, bool]) -> bool:
     if isinstance(stringbool, bool):
         return stringbool  # no conversion needed...
 
-    raise TypeError(f"Could not convert: '{stringbool}' to boolean, " 'only accepts str or boolean')
+    raise TypeError(f"Could not convert: '{stringbool}' to boolean, only accepts str or boolean")
 
 
-def convert_to_fortran_bool(boolean: Union[bool, str]) -> Literal['T', 'F']:
+def convert_to_fortran_bool(boolean: bool | str) -> Literal['T', 'F']:
     """
     Converts a Boolean as string to the format defined in the input
 
@@ -471,10 +470,10 @@ def convert_to_fortran_bool(boolean: Union[bool, str]) -> Literal['T', 'F']:
         raise ValueError(f"A string: {boolean} for a boolean was given, which is not 'True',"
                          "'False', 't', 'T', 'F' or 'f'")
 
-    raise TypeError('convert_to_fortran_bool accepts only a string or ' f'bool as argument, given {boolean} ')
+    raise TypeError(f'convert_to_fortran_bool accepts only a string or bool as argument, given {boolean} ')
 
 
-def convert_fleur_lo(loelements: List[etree._Element]) -> str:
+def convert_fleur_lo(loelements: list[etree._Element]) -> str:
     """
     Converts lo xml elements from the inp.xml file into a lo string for the inpgen
     """
@@ -523,7 +522,7 @@ def convert_fleur_electronconfig(econfig_element: etree._Element) -> str:
     return f'{core_config_str} | {valence_config_str}'
 
 
-def convert_str_version_number(version_str: str) -> Tuple[int, int]:
+def convert_str_version_number(version_str: str) -> tuple[int, int]:
     """
     Convert the version number as a integer for easy comparisons
 
