@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 ###############################################################################
 # Copyright (c), Forschungszentrum Jülich GmbH, IAS-1/PGI-1, Germany.         #
 #                All rights reserved.                                         #
@@ -14,21 +13,23 @@
 This module provides functions to extract distinct parts of the fleur xml files
 for easy versioning and reuse
 """
+from __future__ import annotations
+
 from masci_tools.io.parsers.fleur_schema import schema_dict_version_dispatch
 from masci_tools.io.common_functions import AtomSiteProperties
+from masci_tools.util.typing import XMLLike
+from masci_tools.io.parsers import fleur_schema
+
 from lxml import etree
 import warnings
 import numpy as np
 from logging import Logger
-from typing import TYPE_CHECKING, Iterable, List, Tuple, Union, Dict, Any, Optional
-
-if TYPE_CHECKING:
-    from masci_tools.io.parsers import fleur_schema
+from typing import Any
 
 
-def get_fleur_modes(xmltree: Union[etree._Element, etree._ElementTree],
-                    schema_dict: Union['fleur_schema.InputSchemaDict', 'fleur_schema.OutputSchemaDict'],
-                    logger: Logger = None) -> Dict[str, Any]:
+def get_fleur_modes(xmltree: XMLLike,
+                    schema_dict: fleur_schema.InputSchemaDict | fleur_schema.OutputSchemaDict,
+                    logger: Logger | None = None) -> dict[str, Any]:
     """
     Determine the calculation modes of fleur for the given xml file. Calculation modes
     are things that change the produced files or output in the out.xml files
@@ -167,9 +168,9 @@ def get_fleur_modes(xmltree: Union[etree._Element, etree._ElementTree],
 
 
 @schema_dict_version_dispatch(output_schema=False)
-def get_nkpts(xmltree: Union[etree._Element, etree._ElementTree],
-              schema_dict: Union['fleur_schema.InputSchemaDict', 'fleur_schema.OutputSchemaDict'],
-              logger: Logger = None) -> int:
+def get_nkpts(xmltree: XMLLike,
+              schema_dict: fleur_schema.InputSchemaDict | fleur_schema.OutputSchemaDict,
+              logger: Logger | None = None) -> int:
     """
     Get the number of kpoints that will be used in the calculation specified in the given
     fleur XMl file.
@@ -199,12 +200,12 @@ def get_nkpts(xmltree: Union[etree._Element, etree._ElementTree],
     #Get the name of the current selected kPointSet
     list_name = evaluate_attribute(root, schema_dict, 'listName', logger=logger)
 
-    kpointlists: List[etree._Element] = eval_simple_xpath(root,
-                                                          schema_dict,
-                                                          'kPointList',
-                                                          contains='kPointLists',
-                                                          list_return=True,
-                                                          logger=logger)  #type:ignore
+    kpointlists = eval_simple_xpath(root,
+                                    schema_dict,
+                                    'kPointList',
+                                    contains='kPointLists',
+                                    list_return=True,
+                                    logger=logger)
 
     if len(kpointlists) == 0:
         raise ValueError('No Kpoint lists found in the given inp.xml')
@@ -224,9 +225,9 @@ def get_nkpts(xmltree: Union[etree._Element, etree._ElementTree],
 
 
 @get_nkpts.register(max_version='0.31')
-def get_nkpts_max4(xmltree: Union[etree._Element, etree._ElementTree],
-                   schema_dict: Union['fleur_schema.InputSchemaDict', 'fleur_schema.OutputSchemaDict'],
-                   logger: Logger = None) -> int:
+def get_nkpts_max4(xmltree: XMLLike,
+                   schema_dict: fleur_schema.InputSchemaDict | fleur_schema.OutputSchemaDict,
+                   logger: Logger | None = None) -> int:
     """
     Get the number of kpoints that will be used in the calculation specified in the given
     fleur XMl file. Version specific for Max4 versions or older
@@ -257,23 +258,17 @@ def get_nkpts_max4(xmltree: Union[etree._Element, etree._ElementTree],
     alt_kpt_set = None
     if modes['band'] or modes['gw']:
         expected_mode = 'bands' if modes['band'] else 'gw'
-        alt_kpts: List[etree._Element] = eval_simple_xpath(root,
-                                                           schema_dict,
-                                                           'altKPointSet',
-                                                           list_return=True,
-                                                           logger=logger)  #type:ignore
+        alt_kpts = eval_simple_xpath(root, schema_dict, 'altKPointSet', list_return=True, logger=logger)
         for kpt_set in alt_kpts:
             if evaluate_attribute(kpt_set, schema_dict, 'purpose', logger=logger) == expected_mode:
                 alt_kpt_set = kpt_set
                 break
 
-    kpt_tag: List[etree._Element] = []
+    kpt_tag: list[etree._Element] = []
     if alt_kpt_set is not None:
-        kpt_tag = eval_simple_xpath(alt_kpt_set, schema_dict, 'kPointList', list_return=True,
-                                    logger=logger)  #type:ignore
+        kpt_tag = eval_simple_xpath(alt_kpt_set, schema_dict, 'kPointList', list_return=True, logger=logger)
         if len(kpt_tag) == 0:
-            kpt_tag = eval_simple_xpath(alt_kpt_set, schema_dict, 'kPointCount', list_return=True,
-                                        logger=logger)  #type:ignore
+            kpt_tag = eval_simple_xpath(alt_kpt_set, schema_dict, 'kPointCount', list_return=True, logger=logger)
             if len(kpt_tag) != 0:
                 warnings.warn('kPointCount is not guaranteed to result in the given number of kpoints')
 
@@ -283,14 +278,14 @@ def get_nkpts_max4(xmltree: Union[etree._Element, etree._ElementTree],
                                     'kPointList',
                                     not_contains=['altKPointSet', 'numericalParameters'],
                                     list_return=True,
-                                    logger=logger)  #type:ignore
+                                    logger=logger)
         if len(kpt_tag) == 0:
             kpt_tag = eval_simple_xpath(root,
                                         schema_dict,
                                         'kPointCount',
                                         not_contains='altKPointSet',
                                         list_return=True,
-                                        logger=logger)  #type:ignore
+                                        logger=logger)
             if len(kpt_tag) == 0:
                 raise ValueError('No kPointList or kPointCount found')
             warnings.warn('kPointCount is not guaranteed to result in the given number of kpoints')
@@ -300,17 +295,17 @@ def get_nkpts_max4(xmltree: Union[etree._Element, etree._ElementTree],
                                     'kPointList',
                                     contains='numericalParameters',
                                     list_return=True,
-                                    logger=logger)  #type:ignore
+                                    logger=logger)
 
     nkpts = evaluate_attribute(kpt_tag[0], schema_dict, 'count', logger=logger)
 
     return nkpts
 
 
-def get_cell(xmltree: Union[etree._Element, etree._ElementTree],
-             schema_dict: Union['fleur_schema.InputSchemaDict', 'fleur_schema.OutputSchemaDict'],
-             logger: Logger = None,
-             convert_to_angstroem: bool = True) -> Tuple[np.ndarray, Tuple[bool, bool, bool]]:
+def get_cell(xmltree: XMLLike,
+             schema_dict: fleur_schema.InputSchemaDict | fleur_schema.OutputSchemaDict,
+             logger: Logger | None = None,
+             convert_to_angstroem: bool = True) -> tuple[np.ndarray, tuple[bool, bool, bool]]:
     """
     Get the Bravais matrix from the given fleur xml file. In addition a list
     determining in, which directions there are periodic boundary conditions
@@ -341,8 +336,8 @@ def get_cell(xmltree: Union[etree._Element, etree._ElementTree],
         root = xmltree
     constants = read_constants(root, schema_dict, logger=logger)
 
-    cell: Optional[np.ndarray] = None
-    lattice_tag: Optional[etree._Element] = None
+    cell: np.ndarray | None = None
+    lattice_tag: etree._Element | None = None
     if tag_exists(root, schema_dict, 'bulkLattice', logger=logger):
         lattice_tag = eval_simple_xpath(root, schema_dict, 'bulkLattice', logger=logger)  #type: ignore
         pbc = (True, True, True)
@@ -393,9 +388,9 @@ def get_cell(xmltree: Union[etree._Element, etree._ElementTree],
     return cell, pbc
 
 
-def _get_species_info(xmltree: Union[etree._Element, etree._ElementTree],
-                      schema_dict: Union['fleur_schema.InputSchemaDict', 'fleur_schema.OutputSchemaDict'],
-                      logger: Logger = None) -> Dict[str, Dict[str, str]]:
+def _get_species_info(xmltree: XMLLike,
+                      schema_dict: fleur_schema.InputSchemaDict | fleur_schema.OutputSchemaDict,
+                      logger: Logger | None = None) -> dict[str, dict[str, str]]:
     """
     Gets the species identifiers and information.
     Used to keep species information consistent between
@@ -439,7 +434,7 @@ def _get_species_info(xmltree: Union[etree._Element, etree._ElementTree],
         raise ValueError(
             f'Failed to read in species names and elements. Got {len(names)} names and {len(elements)} elements')
 
-    species_info: Dict[str, Dict[str, str]] = {}
+    species_info: dict[str, dict[str, str]] = {}
     for name, element in zip(names, elements):
         #Check if the species name has a numerical id at the end (separated by - or .)
         #And add all of them first
@@ -465,12 +460,12 @@ def _get_species_info(xmltree: Union[etree._Element, etree._ElementTree],
     return species_info
 
 
-def get_parameter_data(xmltree: Union[etree._Element, etree._ElementTree],
-                       schema_dict: Union['fleur_schema.InputSchemaDict', 'fleur_schema.OutputSchemaDict'],
+def get_parameter_data(xmltree: XMLLike,
+                       schema_dict: fleur_schema.InputSchemaDict | fleur_schema.OutputSchemaDict,
                        inpgen_ready: bool = True,
                        write_ids: bool = True,
                        extract_econfig: bool = False,
-                       logger: Logger = None) -> Dict[str, Any]:
+                       logger: Logger | None = None) -> dict[str, Any]:
     """
     This routine returns an python dictionary produced from the inp.xml
     file, which contains all the parameters needed to setup a new inp.xml from a inpgen
@@ -533,11 +528,7 @@ def get_parameter_data(xmltree: Union[etree._Element, etree._ElementTree],
     parameters['comp'] = filter_out_empty_dict_entries(comp_dict)
 
     # &atoms
-    species_list: List[etree._Element] = eval_simple_xpath(root,
-                                                           schema_dict,
-                                                           'species',
-                                                           list_return=True,
-                                                           logger=logger)  #type:ignore
+    species_list = eval_simple_xpath(root, schema_dict, 'species', list_return=True, logger=logger)
 
     species_info = _get_species_info(xmltree, schema_dict, logger=logger)
 
@@ -565,11 +556,11 @@ def get_parameter_data(xmltree: Union[etree._Element, etree._ElementTree],
 
         if extract_econfig:
             if inpgen_ready:
-                atom_econfig: List[etree._Element] = eval_simple_xpath(species,
-                                                                       schema_dict,
-                                                                       'electronConfig',
-                                                                       list_return=True,
-                                                                       logger=logger)  #type:ignore
+                atom_econfig = eval_simple_xpath(species,
+                                                 schema_dict,
+                                                 'electronConfig',
+                                                 list_return=True,
+                                                 logger=logger)
                 if len(atom_econfig) != 0:
                     atom_dict['econfig'] = convert_fleur_electronconfig(atom_econfig[0])
             else:
@@ -581,8 +572,7 @@ def get_parameter_data(xmltree: Union[etree._Element, etree._ElementTree],
                                                     subtags=True,
                                                     ignore={'flipSpins'})
 
-        atom_lo: List[etree._Element] = eval_simple_xpath(species, schema_dict, 'lo', list_return=True,
-                                                          logger=logger)  #type:ignore
+        atom_lo = eval_simple_xpath(species, schema_dict, 'lo', list_return=True, logger=logger)
 
         if len(atom_lo) != 0:
             atom_dict['lo'] = convert_fleur_lo(atom_lo)
@@ -611,12 +601,12 @@ def get_parameter_data(xmltree: Union[etree._Element, etree._ElementTree],
     # kpt
     if schema_dict.inp_version > (0, 31):
         list_name = evaluate_attribute(root, schema_dict, 'listName', logger=logger)
-        kpointlists: List[etree._Element] = eval_simple_xpath(root,
-                                                              schema_dict,
-                                                              'kPointList',
-                                                              contains='kPointLists',
-                                                              list_return=True,
-                                                              logger=logger)  #type:ignore
+        kpointlists = eval_simple_xpath(root,
+                                        schema_dict,
+                                        'kPointList',
+                                        contains='kPointLists',
+                                        list_return=True,
+                                        logger=logger)
 
         if len(kpointlists) == 0:
             raise ValueError('No Kpoint lists found in the given inp.xml')
@@ -665,13 +655,14 @@ def get_parameter_data(xmltree: Union[etree._Element, etree._ElementTree],
     return parameters
 
 
-def get_structure_data(xmltree: Union[etree._Element, etree._ElementTree],
-                       schema_dict: Union['fleur_schema.InputSchemaDict', 'fleur_schema.OutputSchemaDict'],
-                       include_relaxations: bool = True,
-                       site_namedtuple: bool = True,
-                       convert_to_angstroem: bool = True,
-                       normalize_kind_name: bool = True,
-                       logger: Logger = None) -> Tuple[List[AtomSiteProperties], np.ndarray, Tuple[bool, bool, bool]]:
+def get_structure_data(
+        xmltree: XMLLike,
+        schema_dict: fleur_schema.InputSchemaDict | fleur_schema.OutputSchemaDict,
+        include_relaxations: bool = True,
+        site_namedtuple: bool = True,
+        convert_to_angstroem: bool = True,
+        normalize_kind_name: bool = True,
+        logger: Logger | None = None) -> tuple[list[AtomSiteProperties], np.ndarray, tuple[bool, bool, bool]]:
     """
     Get the structure defined in the given fleur xml file.
 
@@ -734,12 +725,8 @@ def get_structure_data(xmltree: Union[etree._Element, etree._ElementTree],
 
     species_info = _get_species_info(xmltree, schema_dict, logger=None)
 
-    atom_data: List[AtomSiteProperties] = []
-    atom_groups: List[etree._Element] = eval_simple_xpath(root,
-                                                          schema_dict,
-                                                          'atomGroup',
-                                                          list_return=True,
-                                                          logger=logger)  #type:ignore
+    atom_data: list[AtomSiteProperties] = []
+    atom_groups = eval_simple_xpath(root, schema_dict, 'atomGroup', list_return=True, logger=logger)
 
     #Read relaxation information if available
     displacements = None
@@ -761,7 +748,7 @@ def get_structure_data(xmltree: Union[etree._Element, etree._ElementTree],
 
     for indx, group in enumerate(atom_groups):
 
-        atom_positions: List[List[float]] = []
+        atom_positions: list[list[float]] = []
 
         absolute_positions = evaluate_text(group,
                                            schema_dict,
@@ -865,15 +852,15 @@ def get_structure_data(xmltree: Union[etree._Element, etree._ElementTree],
 
 @schema_dict_version_dispatch(output_schema=False)
 def get_kpoints_data(
-    xmltree: Union[etree._Element, etree._ElementTree],
-    schema_dict: Union['fleur_schema.InputSchemaDict', 'fleur_schema.OutputSchemaDict'],
-    name: str = None,
-    index: int = None,
+    xmltree: XMLLike,
+    schema_dict: fleur_schema.InputSchemaDict | fleur_schema.OutputSchemaDict,
+    name: str | None = None,
+    index: int | None = None,
     only_used: bool = False,
-    logger: Logger = None,
+    logger: Logger | None = None,
     convert_to_angstroem: bool = True
-) -> Tuple[Union[List[List[float]], Dict[str, List[List[float]]]], Union[List[float], Dict[str, List[float]]],
-           np.ndarray, Tuple[bool, bool, bool]]:
+) -> tuple[list[list[float]] | dict[str, list[list[float]]], list[float] | dict[str, list[float]], np.ndarray, tuple[
+        bool, bool, bool]]:
     """
     Get the kpoint sets defined in the given fleur xml file.
 
@@ -926,12 +913,12 @@ def get_kpoints_data(
 
     cell, pbc = get_cell(root, schema_dict, logger=logger, convert_to_angstroem=convert_to_angstroem)
 
-    kpointlists: List[etree._Element] = eval_simple_xpath(root,
-                                                          schema_dict,
-                                                          'kPointList',
-                                                          contains='kPointLists',
-                                                          list_return=True,
-                                                          logger=logger)  #type:ignore
+    kpointlists = eval_simple_xpath(root,
+                                    schema_dict,
+                                    'kPointList',
+                                    contains='kPointLists',
+                                    list_return=True,
+                                    logger=logger)
 
     if len(kpointlists) == 0:
         raise ValueError('No Kpoint lists found in the given inp.xml')
@@ -942,13 +929,14 @@ def get_kpoints_data(
             raise ValueError(f'Found no Kpoint list with the name: {name}'
                              f'Available list names: {labels}'
                              'The listName attribute is not consistent with the rest of the input')
-        raise ValueError(f'Found no Kpoint list with the name: {name}' f'Available list names: {labels}')
+        raise ValueError(f'Found no Kpoint list with the name: {name}'
+                         f'Available list names: {labels}')
 
     if index is not None:
         try:
             kpointlists = [kpointlists[index]]
         except IndexError as exc:
-            raise ValueError(f'No kPointList with index {index} found.' f' Only {len(kpointlists)} available') from exc
+            raise ValueError(f'No kPointList with index {index} found. Only {len(kpointlists)} available') from exc
 
     kpoints_data = {}
     weights_data = {}
@@ -983,11 +971,11 @@ def get_kpoints_data(
 
 @get_kpoints_data.register(max_version='0.31')
 def get_kpoints_data_max4(
-        xmltree: Union[etree._Element, etree._ElementTree],
-        schema_dict: Union['fleur_schema.InputSchemaDict', 'fleur_schema.OutputSchemaDict'],
-        logger: Logger = None,
+        xmltree: XMLLike,
+        schema_dict: fleur_schema.InputSchemaDict | fleur_schema.OutputSchemaDict,
+        logger: Logger | None = None,
         convert_to_angstroem: bool = True,
-        only_used: bool = False) -> Tuple[List[List[float]], List[float], np.ndarray, Tuple[bool, bool, bool]]:
+        only_used: bool = False) -> tuple[list[list[float]], list[float], np.ndarray, tuple[bool, bool, bool]]:
     """
     Get the kpoint sets defined in the given fleur xml file.
 
@@ -1026,12 +1014,12 @@ def get_kpoints_data_max4(
 
     cell, pbc = get_cell(root, schema_dict, logger=logger, convert_to_angstroem=convert_to_angstroem)
 
-    kpointlist: List[etree._Element] = eval_simple_xpath(root,
-                                                         schema_dict,
-                                                         'kPointList',
-                                                         list_return=True,
-                                                         not_contains=['altKPoint', 'numericalParameters'],
-                                                         logger=logger)  #type:ignore
+    kpointlist = eval_simple_xpath(root,
+                                   schema_dict,
+                                   'kPointList',
+                                   list_return=True,
+                                   not_contains=['altKPoint', 'numericalParameters'],
+                                   logger=logger)
 
     if len(kpointlist) == 0:
         if getattr(schema_dict, 'out_version', None) is not None:
@@ -1040,7 +1028,7 @@ def get_kpoints_data_max4(
                                            'kPointList',
                                            list_return=True,
                                            contains='numericalParameters',
-                                           logger=logger)  #type:ignore
+                                           logger=logger)
         else:
             raise ValueError('No Kpoint lists found in the given inp.xml')
 
@@ -1063,9 +1051,9 @@ def get_kpoints_data_max4(
 
 
 @schema_dict_version_dispatch(output_schema=False)
-def get_relaxation_information(xmltree: Union[etree._Element, etree._ElementTree],
-                               schema_dict: Union['fleur_schema.InputSchemaDict', 'fleur_schema.OutputSchemaDict'],
-                               logger: Logger = None) -> Dict[str, Any]:
+def get_relaxation_information(xmltree: XMLLike,
+                               schema_dict: fleur_schema.InputSchemaDict | fleur_schema.OutputSchemaDict,
+                               logger: Logger | None = None) -> dict[str, Any]:
     """
     Get the relaxation information from the given fleur XML file. This includes the current
     displacements, energy and posforce evolution
@@ -1111,11 +1099,7 @@ def get_relaxation_information(xmltree: Union[etree._Element, etree._ElementTree
                                               logger=logger)
 
     out_dict['posforces'] = []
-    relax_iters: List[etree._Element] = eval_simple_xpath(relax_tag,
-                                                          schema_dict,
-                                                          'step',
-                                                          list_return=True,
-                                                          logger=logger)  #type:ignore
+    relax_iters = eval_simple_xpath(relax_tag, schema_dict, 'step', list_return=True, logger=logger)
     for step in relax_iters:
         posforces = evaluate_text(step, schema_dict, 'posforce', list_return=True, constants=constants, logger=logger)
         out_dict['posforces'].append(posforces)
@@ -1124,10 +1108,9 @@ def get_relaxation_information(xmltree: Union[etree._Element, etree._ElementTree
 
 
 @get_relaxation_information.register(max_version='0.28')
-def get_relaxation_information_pre029(xmltree: Union[etree._Element, etree._ElementTree],
-                                      schema_dict: Union['fleur_schema.InputSchemaDict',
-                                                         'fleur_schema.OutputSchemaDict'],
-                                      logger: Logger = None) -> None:
+def get_relaxation_information_pre029(xmltree: XMLLike,
+                                      schema_dict: (fleur_schema.InputSchemaDict | fleur_schema.OutputSchemaDict),
+                                      logger: Logger | None = None) -> None:
     """
     Get the relaxation information from the given fleur XML file. This includes the current
     displacements, energy and posforce evolution
@@ -1145,9 +1128,9 @@ def get_relaxation_information_pre029(xmltree: Union[etree._Element, etree._Elem
         f"'get_relaxation_information' is not implemented for inputs of version '{schema_dict['inp_version']}'")
 
 
-def get_symmetry_information(xmltree: Union[etree._Element, etree._ElementTree],
-                             schema_dict: Union['fleur_schema.InputSchemaDict', 'fleur_schema.OutputSchemaDict'],
-                             logger: Logger = None) -> Tuple[List[np.ndarray], List[np.ndarray]]:
+def get_symmetry_information(xmltree: XMLLike,
+                             schema_dict: fleur_schema.InputSchemaDict | fleur_schema.OutputSchemaDict,
+                             logger: Logger | None = None) -> tuple[list[np.ndarray], list[np.ndarray]]:
     """
     Get the symmetry information from the given fleur XML file. This includes the
     rotation matrices and shifts defined in the ``symmetryOperations`` tag.
@@ -1178,8 +1161,7 @@ def get_symmetry_information(xmltree: Union[etree._Element, etree._ElementTree],
     if not tag_exists(root, schema_dict, 'symmetryOperations', logger=logger):
         raise ValueError('No explicit symmetry information included in the given XML file')
 
-    ops: List[etree._Element] = eval_simple_xpath(root, schema_dict, 'symOp', logger=logger,
-                                                  list_return=True)  #type:ignore
+    ops = eval_simple_xpath(root, schema_dict, 'symOp', logger=logger, list_return=True)
 
     rotations = []
     shifts = []
