@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 ###############################################################################
 # Copyright (c), Forschungszentrum Jülich GmbH, IAS-1/PGI-1, Germany.         #
 #                All rights reserved.                                         #
@@ -16,16 +15,21 @@ have the ability to create missing tags on the fly. This functionality is added
 in :py:mod:`~masci_tools.util.xml.xml_setters_xpaths` since we need the schema dictionary
 to do these operations robustly
 """
-from typing import Iterable, Union, List, Any, cast
+from __future__ import annotations
+
+from typing import Iterable, Any, cast
 from lxml import etree
-from masci_tools.util.xml.common_functions import eval_xpath
 import warnings
 
+from masci_tools.util.typing import XPathLike, XMLLike
+from masci_tools.util.xml.common_functions import eval_xpath
+from masci_tools.util.xml.xpathbuilder import XPathBuilder
 
-def xml_replace_tag(xmltree: Union[etree._Element, etree._ElementTree],
-                    xpath: 'etree._xpath',
+
+def xml_replace_tag(xmltree: XMLLike,
+                    xpath: XPathLike,
                     newelement: etree._Element,
-                    occurrences: Union[int, Iterable[int]] = None) -> Union[etree._Element, etree._ElementTree]:
+                    occurrences: int | Iterable[int] | None = None) -> XMLLike:
     """
     replaces xml tags by another tag on an xmletree in place
 
@@ -41,14 +45,15 @@ def xml_replace_tag(xmltree: Union[etree._Element, etree._ElementTree],
     from masci_tools.io.common_functions import is_sequence
 
     if not etree.iselement(xmltree):
-        root = xmltree.getroot()  #type:ignore
+        root = xmltree.getroot()
     else:
         root = xmltree
 
-    nodes: List[etree._Element] = eval_xpath(root, xpath, list_return=True)  #type:ignore
+    nodes: list[etree._Element] = eval_xpath(root, xpath, list_return=True)  #type:ignore
 
     if len(nodes) == 0:
-        warnings.warn(f'No nodes to replace found on xpath: {str(xpath)}')
+        warnings.warn(
+            f'No nodes to replace found on xpath: {str(xpath.path) if isinstance(xpath, XPathBuilder) else str(xpath)}')
 
     if occurrences is not None:
         if not is_sequence(occurrences):
@@ -70,10 +75,10 @@ def xml_replace_tag(xmltree: Union[etree._Element, etree._ElementTree],
     return xmltree
 
 
-def xml_delete_att(xmltree: Union[etree._Element, etree._ElementTree],
-                   xpath: 'etree._xpath',
+def xml_delete_att(xmltree: XMLLike,
+                   xpath: XPathLike,
                    attrib: str,
-                   occurrences: Union[int, Iterable[int]] = None) -> Union[etree._Element, etree._ElementTree]:
+                   occurrences: int | Iterable[int] | None = None) -> XMLLike:
     """
     Deletes an xml attribute in an xmletree.
 
@@ -88,14 +93,16 @@ def xml_delete_att(xmltree: Union[etree._Element, etree._ElementTree],
     from masci_tools.io.common_functions import is_sequence
 
     if not etree.iselement(xmltree):
-        root = xmltree.getroot()  #type:ignore
+        root = xmltree.getroot()
     else:
         root = xmltree
 
-    nodes: List[etree._Element] = eval_xpath(root, xpath, list_return=True)  #type:ignore
+    nodes: list[etree._Element] = eval_xpath(root, xpath, list_return=True)  #type:ignore
 
     if len(nodes) == 0:
-        warnings.warn(f'No nodes to delete attributes on found on xpath: {str(xpath)}')
+        warnings.warn(
+            f'No nodes to delete attributes on found on xpath: {str(xpath.path) if isinstance(xpath, XPathBuilder) else str(xpath)}'
+        )
 
     if occurrences is not None:
         if not is_sequence(occurrences):
@@ -111,9 +118,7 @@ def xml_delete_att(xmltree: Union[etree._Element, etree._ElementTree],
     return xmltree
 
 
-def xml_delete_tag(xmltree: Union[etree._Element, etree._ElementTree],
-                   xpath: 'etree._xpath',
-                   occurrences: Union[int, Iterable[int]] = None) -> Union[etree._Element, etree._ElementTree]:
+def xml_delete_tag(xmltree: XMLLike, xpath: XPathLike, occurrences: int | Iterable[int] | None = None) -> XMLLike:
     """
     Deletes a xml tag in an xmletree.
 
@@ -127,14 +132,15 @@ def xml_delete_tag(xmltree: Union[etree._Element, etree._ElementTree],
     from masci_tools.io.common_functions import is_sequence
 
     if not etree.iselement(xmltree):
-        root = xmltree.getroot()  #type:ignore
+        root = xmltree.getroot()
     else:
         root = xmltree
 
-    nodes: List[etree._Element] = eval_xpath(root, xpath, list_return=True)  #type:ignore
+    nodes: list[etree._Element] = eval_xpath(root, xpath, list_return=True)  #type:ignore
 
     if len(nodes) == 0:
-        warnings.warn(f'No nodes to delete found on xpath: {str(xpath)}')
+        warnings.warn(
+            f'No nodes to delete found on xpath: {str(xpath.path) if isinstance(xpath, XPathBuilder) else str(xpath)}')
 
     if occurrences is not None:
         if not is_sequence(occurrences):
@@ -154,7 +160,7 @@ def xml_delete_tag(xmltree: Union[etree._Element, etree._ElementTree],
     return xmltree
 
 
-def _reorder_tags(node: etree._Element, tag_order: List[str]) -> etree._Element:
+def _reorder_tags(node: etree._Element, tag_order: list[str]) -> etree._Element:
     """
     Order the children of the given node into the given order
 
@@ -189,14 +195,14 @@ def _reorder_tags(node: etree._Element, tag_order: List[str]) -> etree._Element:
     return ordered_node
 
 
-def xml_create_tag(xmltree: Union[etree._Element, etree._ElementTree],
-                   xpath: 'etree._xpath',
-                   element: Union[str, etree._Element],
-                   place_index: int = None,
-                   tag_order: List[str] = None,
-                   occurrences: Union[int, Iterable[int]] = None,
+def xml_create_tag(xmltree: XMLLike,
+                   xpath: XPathLike,
+                   element: str | etree._Element,
+                   place_index: int | None = None,
+                   tag_order: list[str] | None = None,
+                   occurrences: int | Iterable[int] | None = None,
                    correct_order: bool = True,
-                   several: bool = True) -> Union[etree._Element, etree._ElementTree]:
+                   several: bool = True) -> XMLLike:
     """
     This method evaluates an xpath expression and creates a tag in a xmltree under the
     returned nodes.
@@ -234,7 +240,7 @@ def xml_create_tag(xmltree: Union[etree._Element, etree._ElementTree],
     else:
         element_name = element.tag  #type:ignore
 
-    parent_nodes: List[etree._Element] = eval_xpath(xmltree, xpath, list_return=True)  #type:ignore
+    parent_nodes: list[etree._Element] = eval_xpath(xmltree, xpath, list_return=True)  #type:ignore
 
     if len(parent_nodes) == 0:
         raise ValueError(f"Could not create tag '{element_name}' because atleast one subtag is missing. "
@@ -321,12 +327,11 @@ def xml_create_tag(xmltree: Union[etree._Element, etree._ElementTree],
     return xmltree
 
 
-def xml_set_attrib_value_no_create(
-        xmltree: Union[etree._Element, etree._ElementTree],
-        xpath: 'etree._xpath',
-        attributename: str,
-        attribv: Any,
-        occurrences: Union[int, Iterable[int]] = None) -> Union[etree._Element, etree._ElementTree]:
+def xml_set_attrib_value_no_create(xmltree: XMLLike,
+                                   xpath: XPathLike,
+                                   attributename: str,
+                                   attribv: Any,
+                                   occurrences: int | Iterable[int] | None = None) -> XMLLike:
     """
     Sets an attribute in a xmltree to a given value. By default the attribute will be set
     on all nodes returned for the specified xpath.
@@ -344,14 +349,16 @@ def xml_set_attrib_value_no_create(
     from masci_tools.io.common_functions import is_sequence
 
     if not etree.iselement(xmltree):
-        root = xmltree.getroot()  #type:ignore
+        root = xmltree.getroot()
     else:
         root = xmltree
 
-    nodes: List[etree._Element] = eval_xpath(root, xpath, list_return=True)  #type:ignore
+    nodes: list[etree._Element] = eval_xpath(root, xpath, list_return=True)  #type:ignore
 
     if len(nodes) == 0:
-        warnings.warn(f'No nodes to set attribute {attributename} on found on xpath: {str(xpath)}')
+        warnings.warn(
+            f'No nodes to set attribute {attributename} on found on xpath: {str(xpath.path) if isinstance(xpath, XPathBuilder) else str(xpath)}'
+        )
         return xmltree
 
     if occurrences is not None:
@@ -376,10 +383,10 @@ def xml_set_attrib_value_no_create(
     return xmltree
 
 
-def xml_set_text_no_create(xmltree: Union[etree._Element, etree._ElementTree],
-                           xpath: 'etree._xpath',
+def xml_set_text_no_create(xmltree: XMLLike,
+                           xpath: XPathLike,
                            text: Any,
-                           occurrences: Union[int, Iterable[int]] = None) -> Union[etree._Element, etree._ElementTree]:
+                           occurrences: int | Iterable[int] | None = None) -> XMLLike:
     """
     Sets the text of a tag in a xmltree to a given value.
     By default the text will be set on all nodes returned for the specified xpath.
@@ -396,14 +403,16 @@ def xml_set_text_no_create(xmltree: Union[etree._Element, etree._ElementTree],
     from masci_tools.io.common_functions import is_sequence
 
     if not etree.iselement(xmltree):
-        root = xmltree.getroot()  #type:ignore
+        root = xmltree.getroot()
     else:
         root = xmltree
 
-    nodes: List[etree._Element] = eval_xpath(root, xpath, list_return=True)  #type:ignore
+    nodes: list[etree._Element] = eval_xpath(root, xpath, list_return=True)  #type:ignore
 
     if len(nodes) == 0:
-        warnings.warn(f'No nodes to set text on found on xpath: {str(xpath)}')
+        warnings.warn(
+            f'No nodes to set text on found on xpath: {str(xpath.path) if isinstance(xpath, XPathBuilder) else str(xpath)}'
+        )
         return xmltree
 
     if occurrences is not None:
